package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysDictionaryService;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysDictionary;
import cn.estsh.i3plus.pojo.platform.repository.SysDictionaryRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 字典服务接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-28 23:44
 * @Modify :
 **/
@Service
public class SysDictionaryService implements ISysDictionaryService {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysDictionaryService.class);

    @Autowired
    private SysDictionaryRepository sysDictionaryRDao;

    @Override
    public void insertSysDictionary(SysDictionary sysDictionary) {
        long repeatCount = sysDictionaryRDao.findByPropertyCount("dictionaryCode",sysDictionary.getDictionaryCode());
        if (repeatCount > 0){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                    .setErrorDetail("字典代码已存在")
                    .setErrorSolution("请重新输入字典代码")
                    .build();
        }

        // 查询父级字典名称
        if (sysDictionary.getParentId() != null && sysDictionary.getParentId() > 0) {
            LOGGER.info("字典 SYS_DICTIONARY parentId:{}", sysDictionary.getParentId());
            SysDictionary parentSysDictionary = sysDictionaryRDao.getById(sysDictionary.getParentId());
            ValidatorBean.checkNotNull(parentSysDictionary);

            sysDictionary.setRedParentName(parentSysDictionary.getName());
            sysDictionary.setRedParentCode(parentSysDictionary.getDictionaryCode());
        } else {
            sysDictionary.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
        }

        LOGGER.info("字典 SYS_DICTIONARY sysDictionary:{}",sysDictionary);
        sysDictionaryRDao.insert(sysDictionary);
    }

    @Override
    public void deleteSysDictionaryById(String id) {
        LOGGER.info("字典 SYS_DICTIONARY id:{}",id);
        sysDictionaryRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public void updateSysDictionary(SysDictionary sysDictionary) {
        // 校验唯一
        String hqlPack = CoreHqlPack.packHqlSysDictionaryCode(sysDictionary);
        long  repeatCount = sysDictionaryRDao.findByHqlWhereCount(hqlPack);
        if (repeatCount > 0){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("字典代码已存在")
                    .setErrorSolution("请重新输入字典代码")
                    .build();
        }

        // 查询父级字典名称
        if (sysDictionary.getParentId() != null && sysDictionary.getParentId() > 0) {
            LOGGER.info("字典 SYS_DICTIONARY parentId:{}", sysDictionary.getParentId());
            SysDictionary parentSysDictionary = sysDictionaryRDao.getById(sysDictionary.getParentId());
            ValidatorBean.checkNotNull(parentSysDictionary);

            sysDictionary.setRedParentName(parentSysDictionary.getName());
            sysDictionary.setRedParentCode(parentSysDictionary.getDictionaryCode());
        }

        LOGGER.info("字典 SYS_DICTIONARY sysDictionary:{}",sysDictionary);
        sysDictionaryRDao.update(sysDictionary);
    }

    @Override
    public List<SysDictionary> listSysDictionary() {
        LOGGER.info("字典 SYS_DICTIONARY list");
        return sysDictionaryRDao.list();
    }

    @Override
    public SysDictionary getSysDictionaryById(String id) {
        LOGGER.info("字典 SYS_DICTIONARY id:{}",id);
        return sysDictionaryRDao.getById(Long.parseLong(id));
    }

    @Override
    public List<SysDictionary> findSysDictionaryByParentCode(String parentCode) {
        LOGGER.info("字典 SYS_DICTIONARY parentCode:{}",parentCode);
        return sysDictionaryRDao.findByProperty("redParentCode",parentCode);
    }

    @Override
    public ListPager querySysDictionaryByPager(SysDictionary sysDictionary, Pager pager) {
        LOGGER.info("字典 SYS_DICTIONARY SysDictionary:{}，Pager:{}",sysDictionary,pager);
        sysDictionary = sysDictionary == null ? new SysDictionary():sysDictionary;

        String hqlPack = CoreHqlPack.packHqlSysDictionary(sysDictionary);
        pager = PagerHelper.getPager(pager, sysDictionaryRDao.findByHqlWhereCount(hqlPack));
        return new ListPager(sysDictionaryRDao.findByHqlWherePage(hqlPack + sysDictionary.orderBy(),pager),pager);
    }

    @Override
    public List<SysDictionary> findSysDictionaryByTop() {
        LOGGER.info("字典 SYS_DICTIONARY parentId:{}", CommonEnumUtil.PARENT.DEFAULT.getValue());
        return sysDictionaryRDao.findByProperty("parentId",(long)CommonEnumUtil.PARENT.DEFAULT.getValue());
    }

    @Override
    public void deleteSysDictionaryByIds(String[] ids) {
        LOGGER.info("字典 SYS_DICTIONARY ids:{}", ids.toString());
        Long[] array = new Long[ids.length];
        for (int i = 0; i < ids.length; i++) {
            array[i] = Long.parseLong(ids[i]);
        }

        sysDictionaryRDao.deleteByIds(array);
    }

    @Override
    public boolean checkFileName(String fileName) {
        List<SysDictionary> list = findSysDictionaryByParentCode(CommonConstWords.DICTIONARY_FILE_TYPE);

        if(list != null && list.size()> 0){
            String suffix = StringTool.getStringFileSuffix(fileName,true);

            for (SysDictionary dictionary : list) {
                if(suffix.toUpperCase().equals(dictionary.getDictionaryValue().toUpperCase())){
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public SysDictionary getSysDictionaryByParentCodeAndCode(String parentCode, String code) {
        List<SysDictionary> list = sysDictionaryRDao.findByProperty(new String[]{"redParentCode","dictionaryCode"},
                new Object[]{parentCode,code});
        if(list != null && list.size() > 0){
            return list.get(0);
        }
        return null;
    }
}
